/*
** MixerTypes.c
**
** Constructors / destructors for mixer types.
**
** sam-lo_p@epita.fr
*/

#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>
#include<fcntl.h>
#include"MixerTypes.h"
#include"DebugLib.h"



/*#include"Errors.h"
  #include"Charge_Sound.h"
  #include"Constants.h"*/

t_main_mixer* new_main_mixer(dsp, volume, pan, freq, nb_voices)
     t_dsp* dsp;
     int    volume;
     int    pan;
     int    freq;
     int    nb_voices;
{
  t_main_mixer* temp_main_mixer;

  if (dsp == NULL)
    {
      warning("MixerTypes.c: new_main_mixer: You would like to create a t_main_mixer with a NULL dsp!\n");
      return(NULL);
    } /* fi */

  temp_main_mixer = (t_main_mixer*) malloc(sizeof(t_main_mixer));
  temp_main_mixer->dsp             = dsp;
  temp_main_mixer->first_mixer_elt = NULL;
  temp_main_mixer->vol             = volume;
  temp_main_mixer->pan             = pan;
  temp_main_mixer->frq             = freq;
  temp_main_mixer->nb_voices       = nb_voices;
  temp_main_mixer->played_samples  = 0;

  return(temp_main_mixer);
}

int main_mixer_destroy(main_mixer)
     t_main_mixer* main_mixer;
{
  t_mixer_elt* mixer_elt;
  /* ATTENTION ICI LE DSP N'EST PAS LIBERE DE LA MEMOIRE! */
  
  mixer_elt = main_mixer->first_mixer_elt;

  while(mixer_elt != NULL)
    {
      remove_mixer_elt(main_mixer, mixer_elt);
      mixer_elt = main_mixer->first_mixer_elt;
    } /* while */
  free(main_mixer);
  main_mixer = NULL;
  return(0);

} /* main_mixer_destroy */

t_mixer_elt* new_mixer_elt(dsp,tempgmurf,mixer_info)
     t_dsp*        dsp;
     t_tempgmurf*  tempgmurf;
     t_mixer_info* mixer_info;
{
  t_mixer_elt* temp_mixer_elt;

  temp_mixer_elt = (t_mixer_elt*) malloc (sizeof(t_mixer_elt));

  temp_mixer_elt->dsp            = dsp;
  temp_mixer_elt->tempgmurf      = tempgmurf;
  temp_mixer_elt->mixer_info     = mixer_info;
  temp_mixer_elt->next_mixer_elt = NULL;
  temp_mixer_elt->rec_or_play    = PLAY;
  temp_mixer_elt->mute           = FALSE_;
  temp_mixer_elt->finished       = FALSE_;

  return temp_mixer_elt;
} /* new_mixer_elt */

t_mixer_elt* add_mixer_elt(main_mixer,dsp, tempgmurf,mixer_info)
     t_dsp*        dsp;
     t_main_mixer* main_mixer;
     t_tempgmurf*  tempgmurf;
     t_mixer_info* mixer_info;
{
  t_mixer_elt* firstmixer;
  t_mixer_elt* tempmixerelt;

  if (tempgmurf == NULL)
    warning("MixerTypes.c: add_mixer_elt: Creating a t_mixer_elt with a "
	    "NULL t_tempgmurf");
  if (mixer_info == NULL)
    warning("MixerTypes.c: add_mixer_elt: Creating a t_mixer_elt with a "
	    "NULL t_mixer_info");
  
  if (main_mixer == NULL)
    {
      warning("MixerTypes.c: add_mixer_elt: you would like to add a mixer "
	      "element to a NULL main_mixer!");
      return (NULL);
    } /* fi */

  firstmixer = main_mixer->first_mixer_elt;
  if (firstmixer != NULL)
    {
      while(firstmixer->next_mixer_elt != NULL) /* parcours */
	{
	  firstmixer = firstmixer->next_mixer_elt;
	} /* while */
/*      tempmixerelt = (t_mixer_elt*) malloc(sizeof(t_mixer_elt));
	tempmixerelt->dsp = dsp;
	tempmixerelt->tempgmurf = tempgmurf;	
	tempmixerelt->mixer_info = mixer_info;*/	
      tempmixerelt = new_mixer_elt(dsp, tempgmurf, mixer_info);
      
      tempmixerelt->next_mixer_elt = NULL;
      firstmixer->next_mixer_elt = tempmixerelt;
    }
  else
    {
/*      tempmixerelt = (t_mixer_elt*) malloc(sizeof(t_mixer_elt));
	tempmixerelt->dsp = dsp;	
	tempmixerelt->tempgmurf = tempgmurf;
	tempmixerelt->mixer_info = mixer_info;*/
      tempmixerelt = new_mixer_elt(dsp, tempgmurf, mixer_info);
      tempmixerelt->next_mixer_elt = NULL;
      main_mixer->first_mixer_elt = tempmixerelt;
    } /* if */
  return (tempmixerelt);

} /* add_mixer_elt */

int free_mixer_elt(tempmxelt)
     t_mixer_elt* tempmxelt;
{
  t_point*     tempcurve;
  t_instrux*   tempinstrux;
  
  if (tempmxelt == NULL)
    {
      warning("MixerTypes.c: free_mixer_elt: You would like to free a NULL "
	      "t_mixer_elt!");
      return(-1);
    }

  if (tempmxelt->mixer_info == NULL)
    {
      warning("MixerTypes.c: free_mixer_elt: I won't free a NULL t_mixer_info!");
    }
  else
    {
      while (tempmxelt->mixer_info->vol_curve != NULL) /* vol_curve */
	{
	  tempcurve = tempmxelt->mixer_info->vol_curve->next_point;
	  free(tempmxelt->mixer_info->vol_curve);
	  tempmxelt->mixer_info->vol_curve = tempcurve;
	}
      
      while (tempmxelt->mixer_info->pan_curve != NULL) /* pan_curve */
	{
	  tempcurve = tempmxelt->mixer_info->pan_curve->next_point;
	  free(tempmxelt->mixer_info->pan_curve);
	  tempmxelt->mixer_info->pan_curve = tempcurve;
	}
      
      while (tempmxelt->mixer_info->frq_curve != NULL) /* frq_curve */
	{
	  tempcurve = tempmxelt->mixer_info->frq_curve->next_point;
	  free(tempmxelt->mixer_info->frq_curve);
	  tempmxelt->mixer_info->frq_curve = tempcurve;
	}
      
      while (tempmxelt->mixer_info->first_instrux != NULL) /* first_instrux */
	{
	  tempinstrux = tempmxelt->mixer_info->first_instrux->next_instrux;
	  free(tempmxelt->mixer_info->first_instrux);
	  tempmxelt->mixer_info->first_instrux = tempinstrux;
	}      
      free(tempmxelt->mixer_info);
    } /* fi */

  if (tempmxelt->tempgmurf == NULL)
    warning("MixerTypes.c: free_mixer_elt: I won't free a NULL t_tempgmurf!");
  else
    {  
      // If the waveditor is not closed, we dont free the tempgmurf
      if ((tempmxelt->tempgmurf->connected == IN_WE) || (tempmxelt->tempgmurf->connected == IN_BOTH))
	{
	  warning("MixerTypes.c: free_mixer_elt: t_temgmurf is already connected "
		  "by the Waveditor. It's OK, I won't free nor close it.");
	}
      else
	{
	  close(tempmxelt->tempgmurf->file);
	  free(tempmxelt->tempgmurf);
	}
    } /* fi */

  free(tempmxelt);
  
  return(0);

} /* free_mixer_elt */
 
int remove_mixer_elt(mmx, elt)
     t_main_mixer* mmx; /* main mixer */
     t_mixer_elt*  elt; /* element to remove */
{
  t_mixer_elt* temp_elt;
  t_mixer_elt* prev_elt;

  if (mmx == NULL)
    {
      warning("MixerTypes.c: remove_mixer_elt: You would like to remove a "
	      "t_mixer_elt from a NULL t_main_mixer!");
      return(-1);
    }

  if (elt == NULL)
    {
      warning("MixerTypes.c: remove_mixer_elt: You would like to remove a "
	      "NULL t_mixer_elt from the t_main_mixer!");
      return(-2);
    }

  close(elt->tempgmurf->file);
  unlink(elt->tempgmurf->tempname);
  
  if (mmx->first_mixer_elt == elt)
    {
      mmx->first_mixer_elt = elt->next_mixer_elt;
      free_mixer_elt(elt);
      return(0);
    }

  temp_elt = mmx->first_mixer_elt->next_mixer_elt;
  prev_elt = mmx->first_mixer_elt;
  while ((temp_elt != elt) && (temp_elt != NULL))
    {
      prev_elt = temp_elt;
      temp_elt = temp_elt->next_mixer_elt;
    } /* while */
  if (temp_elt == NULL)
    {
      warning("MixerTypes.c: remove_mixer_elt: I can't find the t_mixer_elt "
	      "you wanted me to remove.");
      return(-1);
    } /* fi */
  
  prev_elt->next_mixer_elt = temp_elt->next_mixer_elt;
  free_mixer_elt(temp_elt);
  return(0);

} /* remove_mixer_elt */

t_mixer_info* new_mixer_info(volume, freq_on, panning, begin_date)
     float volume;     /* volume de l'onde */
     BOOL freq_on;       /* frequence */
     float panning;    /* balance */
     int begin_date; /* Date de dpart */

{
  t_mixer_info* mixer_info;
  mixer_info = (t_mixer_info*) malloc(sizeof(t_mixer_info));

  mixer_info->begin_date    = begin_date;
  mixer_info->volume        = volume;
  mixer_info->do_freq       = freq_on;
  mixer_info->panning       = panning;
  mixer_info->vol_curve     = NULL;
  mixer_info->frq_curve     = NULL;
  mixer_info->pan_curve     = NULL;
  mixer_info->first_instrux = NULL;
  return(mixer_info);

}

t_tempgmurf* new_tempgmurf(name, origname) /* TEMPORAIRE! Procdure de test! */
     char* name;
     char* origname;
{
  t_tempgmurf* tempgmurf;

  tempgmurf = (t_tempgmurf *) malloc(sizeof(t_tempgmurf));
  tempgmurf->file = open(name, 
			 O_RDWR | O_CREAT | O_APPEND | O_NONBLOCK, 
			 0644);
  if (tempgmurf->file == -1)
    {
      free(tempgmurf);
      warning("MixerTypes.c: new_tempgmurf: Erreur lors de l'ouverture du fichier temporaire");
      return (NULL);
    }
  
  strcpy(tempgmurf->tempname,name);
  if (origname == NULL)
    strcpy(tempgmurf->origname,NEW_TMPGMURF_NAME);
  else
    strcpy(tempgmurf->origname, origname);
  tempgmurf->position        = 0;
  tempgmurf->temp_size       = 0;
  tempgmurf->bits_per_sample = DEF_BPS;
  tempgmurf->chanels         = DEF_CHNS;
  tempgmurf->sample_rate     = DEF_RATE;
  
  /*  tempgmurf = charge_sound_file("/tmp/Gmurf/toto", name, 
      44100);*/
  //  close(tempgmurf->file);
  
  /* NE PAS OUBLIER ORIGNAME */
  return (tempgmurf);
} /* new_tempgmurf */

